#!/bin/bash

PARAMFILE=${1}

. ${PARAMFILE}

extractVideo () {
  # Split the portion of the video that we want to work on directly from the source file
  # converting the video to frame files one PNG file per frame and
  # splitting the audio to to an mp3 file
  rm -fr "${OUTPUTFILE}-frames"
  mkdir -p "${OUTPUTFILE}-frames"

  ffmpeg -loglevel quiet -threads 4 \
         -i ${INPUTFILE} -ss ${SCENESTART} -t ${SCENEDURATION} \
         -f image2 -y "${OUTPUTFILE}-frames"/frame%d.png \
         -acodec copy -sameq -y "${OUTPUTFILE}.mp3"

}


createIntro () {
  # Create the introduction image by merging the logo file with the titles for the video
  convert ${LOGO} -gravity Center -font DejaVu-Sans-Book \
         -pointsize 20 -fill gray -draw "text 1,21 'K/W Linux users group'" \
         -fill white -draw "text 0,20 'K/W Linux users group'" \
         -pointsize 50 -fill gray -draw "text 2,72 '${SPEAKER}'" \
         -fill white -draw "text 0,70 '${SPEAKER}'" \
         -pointsize 30 -fill gray -draw "text 1,131 '${TITLE}'" \
         -fill white -draw "text 0,130 '${TITLE}'" \
         -pointsize 20 -fill gray -draw "text 1,171 '${DATE}'" \
         -fill white -draw "text 0,170 '${DATE}'" \
         "${OUTPUTFILE}-intro.png"

  # Create still video from the introduction image and silence
  ffmpeg -loglevel quiet -threads 4 \
         -loop_input -i "${OUTPUTFILE}-intro.png" -qscale 1 -r 29.97 -t ${LOGOLENGHT} \
         -ar 48000 -t ${LOGOLENGHT} -f s16le -acodec pcm_s16le -i /dev/zero -ab 64K -f mp2 -acodec mp2 \
         -map 0.0 -map 1.0 -sameq -f mpegts -y "${OUTPUTFILE}-intro.mpg"

}



createExit () {
  # Merge the logo file with the closing for the video
  convert ${LOGO}  -gravity Center -font DejaVu-Sans-Book \
         -pointsize 20 -fill gray -draw "text 1,21 'K/W Linux users group'" \
         -fill white -draw "text 0,20 'K/W Linux users group'" \
         -pointsize 20 -fill gray -draw "text 1,71 'KWLUG logo and name are'" \
         -fill white -draw "text 0,70 'KWLUG logo and name are'" \
         -pointsize 20 -fill gray -draw "text 1,101 'trademarks of KWLUG'" \
         -fill white -draw "text 0,100 'trademarks of KWLUG'" \
         -pointsize 20 -fill gray -draw "text 1,141 'http://www.kwlug.org'" \
         -fill white -draw "text 0,140 'http://www.kwlug.org'" \
         "${OUTPUTFILE}-exit.png"


  # Create still video from the exit image and silence
  ffmpeg -loglevel quiet -threads 4 \
         -loop_input -i "${OUTPUTFILE}-exit.png" -qscale 1 -r 29.97 -t ${LOGOLENGHT} \
         -ar 48000 -t ${LOGOLENGHT} -f s16le -acodec pcm_s16le -i /dev/zero -ab 64K -f mp2 -acodec mp2 \
         -map 0.0 -map 1.0 -sameq -f mpegts -y "${OUTPUTFILE}-exit.mpg"


}


fadeOutIntro () {
  # Fadeout intro into the first 3 seconds of the video (3 seconds at 29.97 frames per second ~ 90 )
  mkdir -p "${OUTPUTFILE}-frames2"
  for i in {1..90}; do 
    percent=$(echo "scale=3; ${i}*100/90" | bc )
    convert -compose dissolve -gravity center -define compose:args=${percent} \
            -composite "${OUTPUTFILE}-intro.png" "${OUTPUTFILE}-frames"/frame${i}.png "${OUTPUTFILE}-frames2"/frame${i}.png
  done
}

reasembleVideo () {
  # Merge the modified frames onto the original frames
  cp "${OUTPUTFILE}-frames2"/frame*.png "${OUTPUTFILE}-frames"

  # Reassembles the modified frames and the audio into an output video
  ffmpeg -loglevel quiet -threads 4 \
         -r 29.97 -f image2 -i "${OUTPUTFILE}-frames"/frame%d.png \
         -i "${OUTPUTFILE}.mp3" -acodec copy \
         -map 0.0 -map 1.0 -sameq -f mpegts -y "${OUTPUTFILE}.mpg"

  ffmpeg -threads 4 \
         -i concat:"${OUTPUTFILE}-intro.mpg"\|"${OUTPUTFILE}.mpg"\|"${OUTPUTFILE}-exit.mpg" \
         -r 29.97 -sameq -y "${OUTPUTFILE}.mp4"

}


overlayImage () {
  # Overlays an image on top of each frame in a range.
  FILE="${1}"
  GRAVITY=$2
  START_FRAME=$(echo "scale=0; ${3}*29.97/1" | bc )
  END_FRAME=$(echo "scale=0; ${4}*29.97/1" | bc )

  for (( i=${START_FRAME} ; i<=${END_FRAME} ; i++ )) ; do 
    convert -compose dissolve -gravity ${GRAVITY} -define compose:args=90 \
            -composite "${OUTPUTFILE}-frames"/frame${i}.png "${FILE}" "${OUTPUTFILE}-frames2"/frame${i}.png
  done
}

# These functions do not depend on each other. They can be executed in parallel
extractVideo &
createIntro &
createExit &
echo processing
wait

echo "Creating overlays"

# These functions do not depend on each other. They can be executed in parallel
fadeOutIntro 

# Executes the "overlays" function defined in the parameters file
overlays

reasembleVideo

# Cleanup
rm -fr "${OUTPUTFILE}-frames" "${OUTPUTFILE}-frames2" 
rm "${OUTPUTFILE}-intro.png" "${OUTPUTFILE}-exit.png" "${OUTPUTFILE}.mp3" "${OUTPUTFILE}.mpg" "${OUTPUTFILE}-intro.mpg" "${OUTPUTFILE}-exit.mpg"


# Done!
echo "Done!"


